import useSWR from 'swr';
import config from './config';

const endpoint = config.endpoint;

interface IDatamanage {
  tables: any;
  loading: boolean;
  error?: any;
  refresh: () => void;
}

export const getDataMgmtList = (
  user_id: string,
  project_id: string
): IDatamanage => {
  const url = `${endpoint}/jpt/datamanagement/?user_id=${user_id}&project_id=${project_id}`;
  const { data, error, mutate } = useSWR(url, (...args) =>
    fetch(...args).then(res => res.json())
  );
  return {
    tables: data?.data,
    loading: !error && !data,
    error: error || data?.code !== 200,
    refresh: mutate
  };
};

export const fetchFile = async (uri: string): Promise<any> => {
  const url = `${endpoint}/jpt/files/?uri=${uri}`;
  try {
    const resp = await fetch(url, { method: 'get' });
    if (!resp.ok) {
      return 'error';
    }
    const data = await resp.text();
    return data;
  } catch (e) {
    console.log(e);
    return 'error';
  }
};

export const deleteDataMgmt = async (id: string): Promise<any> => {
  const url = `${endpoint}/jpt/datamanagement/?data_management_id=${id}`;
  try {
    const resp = await fetch(url, { method: 'delete' });
    if (!resp.ok) {
      return 'error';
    }
    const data = await resp.json();
    if (data.code !== 200) {
      return 'error';
    }
    return data.data;
  } catch (e) {
    console.log(e);
    return 'error';
  }
};
