import React from 'react';
import { deleteDataMgmt, getDataMgmtList } from './api/datamanage';
import Menu from '@mui/material/Menu';
import MenuItem from '@mui/material/MenuItem';
import fileIcon from '../style/img/table.png';
interface IProps {
  onOpenData: (tableInfo: any) => void;
}

const user_id = 'test';
const project_id = 'test';

const DataManage: React.FC<IProps> = props => {
  const [contextMenu, setContextMenu] = React.useState<{
    mouseX: number;
    mouseY: number;
  } | null>(null);

  const [currentTable, setCurrentTable] = React.useState<any | null>(null);

  const { tables, loading, error, refresh } = getDataMgmtList(
    user_id,
    project_id
  );

  const tableClick = (table: any) => {
    props.onOpenData(table);
  };

  const handleContextMenu = (event: React.MouseEvent, table: any) => {
    event.preventDefault();
    setContextMenu(
      contextMenu === null
        ? {
            mouseX: event.clientX + 2,
            mouseY: event.clientY - 6
          }
        : // repeated contextmenu when it is already open closes it with Chrome 84 on Ubuntu
          // Other native context menus might behave different.
          // With this behavior we prevent contextmenu from the backdrop to re-locale existing context menus.
          null
    );
    setCurrentTable(table);
  };

  const handleClose = () => {
    setContextMenu(null);
  };

  const handleWatch = () => {
    setContextMenu(null);
    props.onOpenData(currentTable);
  };

  const handleDelete = async () => {
    setContextMenu(null);
    await deleteDataMgmt(currentTable.id);
    refresh();
  };

  const contextMenuView = (
    <Menu
      open={contextMenu !== null}
      onClose={handleClose}
      anchorReference="anchorPosition"
      anchorPosition={
        contextMenu !== null
          ? { top: contextMenu.mouseY, left: contextMenu.mouseX }
          : undefined
      }
    >
      <MenuItem onClick={handleWatch}>查看</MenuItem>
      <MenuItem onClick={handleDelete}>删除</MenuItem>
    </Menu>
  );

  let el: JSX.Element[] | JSX.Element;
  if (error) {
    el = <div className="datamanage-label">Error</div>;
  } else if (loading) {
    el = <div className="datamanage-label">Loading ...</div>;
  } else {
    el = tables.map((item: any) => {
      return (
        <div
          className="datamanage-label"
          onDoubleClick={() => tableClick(item)}
          onContextMenu={evt => handleContextMenu(evt, item)}
          key={item.id}
        >
          <img className="datamanage-img" src={fileIcon} alt="" />
          <span>{item.name}</span>
        </div>
      );
    });
  }

  return (
    <div className="datamanage">
      {el}
      {contextMenuView}
    </div>
  );
};

export default DataManage;
