import React from 'react';
import { ReactWidget } from '@jupyterlab/apputils';
import { Widget } from '@lumino/widgets';
import { DagDataView } from './dataview';
import { fetchFile } from './api/datamanage';

class DataTableWidget extends ReactWidget {
  constructor(table: any, options?: Widget.IOptions) {
    super(options);
    void this._requestData(table);
    this._table = table;
  }

  private _requestData = async (table: any) => {
    const res = await fetchFile(table.storage_path);
    if (res === 'error') {
      this._err = res;
    } else {
      const res_data = res.split('\n');
      const data = res_data.filter((item: string) => item !== '');
      let fields: string[] = [];
      let dataList: any = [];
      data.forEach((item: string, index: number) => {
        const list: string[] = item.split(',');
        if (index === 0) {
          fields = list;
        } else {
          dataList.push({ ...list });
        }
      });
      this._data = { fields, dataList };
    }
    this.update();
  };

  render(): JSX.Element {
    if (this._err) {
      return <div>Error</div>;
    } else if (!this._data) {
      return <div>Loading ...</div>;
    } else {
      return <DagDataView data={this._data} info={this._table} />;
    }
  }

  private _data: any | null = null;
  private _err: any;
  private _table: any;
}

export default DataTableWidget;
