import React from 'react'
import { MinusCircleOutlined, PlusCircleOutlined, EditOutlined } from '@ant-design/icons';
import { Node } from '@antv/x6';
import { Button, Form, Input, Space, Modal, Collapse, Table, Empty, message} from 'antd';
import type { ColumnsType } from 'antd/es/table';
import ScriptEditor from './ScriptEditor';

const { Panel } = Collapse;

interface DataType {
  key: React.Key;
  name: string;
  field1: string;
  field2: string;
  field3: string;
  field4: string;
  field5: string;
  field6: string;
  field7: string;
  field8: string;
  field9: string;
}

const columns: ColumnsType<DataType> = [
  {
    title: 'Name',
    width: 100,
    dataIndex: 'name',
    key: 'name',
    fixed: 'left',
  },
  {
    title: 'String',
    children: [
      {
        title: 'field1',
        dataIndex: 'field1',
        key: '1',
         width: 150,
      },
      {
        title: 'field2',
        dataIndex: 'field2',
        key: '2',
         width: 150,
      },
      {
        title: 'field3',
        dataIndex: 'field3',
        key: '3',
         width: 150,
      },
    ]
  },
  {
    title: 'Int',
    children: [
      {
        title: 'field4',
        dataIndex: 'field4',
        key: '4',
         width: 150,
      },
      {
        title: 'field5',
        dataIndex: 'field5',
        key: '5',
         width: 150,
      },
      {
        title: 'field6',
        dataIndex: 'field6',
        key: '6',
         width: 150,
      },
    ]
  },
  {
    title: 'Text',
    children: [
      {
        title: 'field7',
        dataIndex: 'field7',
        key: '7',
         width: 150,
      },
      {
        title: 'field8',
        dataIndex: 'field8',
        key: '8',
         width: 150,
      },
      {
        title: 'field9',
        dataIndex: 'field9',
        key: '9',
         width: 150,
      },
    ]
  },
];

const data: DataType[] = [];
// for (let i = 0; i < 100; i++) {
//   data.push({
//     key: i,
//     name: `data ${i}`,
//     field1: 'field1',
//     field2: 'field2',
//     field3: 'field3',
//     field4: 'field4',
//     field5: 'field5',
//     field6: 'field6',
//     field7: 'field7',
//     field8: 'field8',
//     field9: 'field9',
//   });
// }

export default class ScriptNodeInfo extends React.Component<any, any> {

  constructor(props: any) {
    super(props)
    this.state = {
      scriptModalVisible: false,
      checking: false,
      inputNumber: 0,
      outputData: [],
      outputEditing: false
    }
  }
  
  editorRef = React.createRef()

  showScriptModal = () => {
    this.setState({scriptModalVisible: true})
  }

  // 保存编辑的代码
  submitScriptEdit = () => {
    const editorValue = (this.editorRef.current as any).state.editor.getValue()
    this.props.nodeInfo.scriptText = editorValue
    this.setState({scriptModalVisible: false})
  }

  // 提交代码校验
  submitSyntaxCheck = async () => {
    this.setState({checking: true});
    await (this.editorRef.current as any).checkSyntax()
    this.setState({checking: false});
  }

  componentDidMount(): void {
    this.setState({inputNumber: this.props.nodeInfo.inputNumber, outputData: this.props.nodeInfo.outputData})
  }

  // 添加输入端点
  updateInputPorts = (node: Node, number: number) => {
    const currentInputPortsNumber = node.getPorts().reduce((pre: number, item: any) => {
      return item.group === 'top' ? pre + 1 : pre
    }, 0)
    for (let i = number; i < currentInputPortsNumber; i++) {
      node.removePortAt(number)
    }
    for (let i = currentInputPortsNumber; i < number; i++) {
      node.addPort({group: 'top', id: `input${i}`})
    }
  }

  // 更新输入数量
  updateNumber = (inputNumber: number) => {
    const id = this.props.nodeInfo.id
    const node = this.props.graph.getCellById(id)
    this.setState({inputNumber})
    this.props.nodeInfo.inputNumber = inputNumber
    this.updateInputPorts(node, inputNumber)
  }

  updateOutputPorts = (outputData: any) => {
    const id = this.props.nodeInfo.id
    const node = this.props.graph.getCellById(id)
    const currentOutputPorts = node.getPorts().reduce((pre: any, item: any) => {
      return item.group === 'bottom' ? pre.concat(item) : pre
    }, [])
    currentOutputPorts.forEach((item: any) => {
      const index = outputData.findIndex((output: any) => output.outputVar === item.id)
      if (index === -1) {
        node.removePort(item.id)
      }
    })
    outputData.forEach((item: any) => {
      const index = currentOutputPorts?.findIndex((port: any) => port.id === item.outputVar)
      if (index === -1) {
        node.addPort({group: 'bottom', id: item.outputVar})
      }
    })
  }

  enterOutputVar = (e: any) => {
    const value = e.target.value.trim()
    if(e.key === 'Enter' && value) {
      // e.target.setAttribute('disabled', 'disabled')
      this.setState({outputEditing: false})
      // const outputData = values.outputData?.filter((item: any) => {
      //   return item && item.outputVar
      // })
      // this.setState({outputData})
      const outputData = this.state.outputData
      outputData.push({outputVar: value})
      this.setState({outputData})
      this.props.nodeInfo.outputData = outputData
      this.updateOutputPorts(outputData)
    }
  }

  render() {
    return (
      <div className='node-script'>
        {/* 脚本节点信息 */}
        <Form
          name="datasourceInfo"
          autoComplete="off"
          layout='vertical'
          initialValues={{
            nodeId: this.props.nodeInfo.nodeId,
            paramText: this.props.nodeInfo.paramText,
            nodeName: this.props.nodeInfo.nodeName,
            outputData: this.props.nodeInfo.outputData
          }}
          onValuesChange={(changedValue, values) => {
            const paramText = values.paramText
            const nodeName = values.nodeName
            this.props.nodeInfo.paramText = paramText
            this.props.nodeInfo.nodeName = nodeName
          }}
        >
          <Form.Item name='nodeId' label="节点ID">
            <Input disabled/>
          </Form.Item>
          <Form.Item name='nodeName' label="节点名称:">
            <Input placeholder='请输入节点名称'/>
          </Form.Item>
          <p>输入:</p>
          { 
            Array.from({ length: this.state.inputNumber }, (_, i) => (
              <div style={{display: 'flex', alignItems: 'center', columnGap: '2px', width: '30%', marginBottom: '5px'}} key={i}>
                <Input disabled style={{width: '70%'}} defaultValue={`input`+ i}/>
                { i === this.state.inputNumber - 1 && 
                  <MinusCircleOutlined
                    onClick={() => {
                      this.updateNumber(this.state.inputNumber - 1)
                    }}
                    style={{color:'#147BD1'}}
                  />
                }
              </div>
            ))
          }
          <Button
            type="link"
            onClick={ () => {
              this.updateNumber(this.state.inputNumber + 1)
            }} icon={<PlusCircleOutlined />} style={{paddingLeft: 0, paddingRight: 0}}>
            添加输入
          </Button>
          <Form.Item name='paramText' label="参数文本:">
            <Input.TextArea placeholder='# input will be transfered into context_string'/>
          </Form.Item>
          <Form.Item label="脚本编辑:">
            <Button type="link"
              onClick={this.showScriptModal}
              icon={<EditOutlined />}
              style={{paddingLeft: 0, paddingRight: 0}}
            >
              编辑脚本
            </Button>
          </Form.Item>
          <p>输出结果:</p>
          <Form.List name="outputData">
            {(fields, { add, remove }) => (
              <>
                {fields.map(({ key, name, ...restField }) => (
                  <Space key={key} style={{ display: 'flex', padding: '10px 10px 0', background: '#EDF0F6', marginBottom: '2px' }} align="baseline">
                    <Form.Item
                      label={'输出结果'}
                      {...restField}
                      name={[name, 'outputVar']}
                    >
                      {/* <Input placeholder='请输入输出变量' onKeyUp={this.enterOutputVar}/>  */}
                      { this.state.outputEditing && name === this.state.outputData.length ? 
                        <Input placeholder='请输入输出变量' onKeyUp={this.enterOutputVar}/> :
                        <Input placeholder='请输入输出变量' onKeyUp={this.enterOutputVar} disabled style={ {color: 'black', border: 'none', background: '#edf0f6'}}/>
                      }
                      
                    </Form.Item>
                    <MinusCircleOutlined onClick={() => {
                      const outputData = this.state.outputData
                      outputData.splice(name, 1)
                      this.updateOutputPorts(outputData)
                      return remove(name)
                    }} 
                      style={{color:'#147BD1'}}/>
                  </Space>
                ))}
                { !(this.props.nodeInfo.label === 'sql' && fields.length > 0) &&
                  <Form.Item>
                    <Button type="link" onClick={() => {
                      if (this.state.outputEditing) {
                        message.warning('请先确认前一个变量!')
                        return
                      }
                      this.setState({outputEditing: true})
                      return add()
                    }} icon={<PlusCircleOutlined />} style={{paddingLeft: 0, paddingRight: 0}}>
                      添加输出
                    </Button>
                  </Form.Item>
                }
              </>
            )}
          </Form.List>
        </Form>
        {/* 脚本编辑弹窗 */}
        <Modal
          title="编辑脚本"
          open={this.state.scriptModalVisible}
          onOk={this.submitScriptEdit}
          className='script-modal'
          destroyOnClose={true}
          width={'90%'}
          style={{ top: 20 }}
          onCancel={() => {this.setState({scriptModalVisible: false})}}
          footer={[
            <Button key="back" onClick={() => {this.setState({scriptModalVisible: false})}}>取消</Button>,
            <Button loading={this.state.checking} key="check" type='primary' onClick={() => {this.submitSyntaxCheck()}}>代码校验</Button>,
            <Button key="submit" type='primary' onClick={this.submitScriptEdit}>确定修改</Button>
          ]}
        >
          <div className='modal-container'>
            {/* 数据表展示 */}
            <div className='modal-table'>
              { this.state.inputNumber === 0 ?
                <Empty
                  style={{
                    margin: '50% 0',
                    transform: 'translate(0, -50%)'
                  }}
                  image={Empty.PRESENTED_IMAGE_SIMPLE}
                  description={
                    <span>暂无输入数据</span>
                  }
                  /> :
                <Collapse accordion>
                  {Array.from({length: this.state.inputNumber}, (_, i) => (
                    <Panel header={`input${i}`} key={i} style={{position: 'relative', paddingTop: 0}}>
                      <div style={{position: 'absolute', right: 0, left: 0}}>
                        <Table showHeader={data?.length !== 0} bordered columns={columns} dataSource={data} scroll={{ x: 1500, y: 400 }} />
                      </div>
                    </Panel>
                  ))}
                  {/* <Panel header="input1" key="1" style={{position: 'relative', paddingTop: 0}}>
                    
                  </Panel>
                  <Panel header="input2" key="2"></Panel>
                  <Panel header="input3" key="3"></Panel> */}
                </Collapse>
              }
            </div>
            {/* 脚本编辑 */}
            <div className='modal-script'>
              <div className='script-header'></div>
              <ScriptEditor
                inputNumber={this.state.inputNumber}
                type={this.props.nodeInfo.label}
                scriptText={this.props.nodeInfo.scriptText}
                ref={this.editorRef as any}
              />
            </div>
          </div>
        </Modal>
      </div>
    )
  }
}